/*******************************************************************************
 *	Process.h		Processerate output
 *			T.Barnaby,	BEAM Ltd,	2011-07-05
 *******************************************************************************
 */
#ifndef GEN_HH
#define GEN_HH

#include <stdio.h>
#include <BError.h>
#include <BDict.h>
#include <BFile.h>
#include <Main.h>
#include <Node.h>
#include <BdsD.h>
#include <BdsC.h>

using namespace Bds;

class Process {
public:
			Process(AdminAccess& bds, Parameters& params);
			~Process();

	BError		init();
	BError		run(Node* n);
	BUInt		line();

private:
	BError				cmdAdd(Node* n);
	BError				cmdRead(Node* n);
	BError				cmdFind(Node* n);
	
	template <typename T> BError	cmdInfo(Node* n);
	BError				cmdInfo(Node* n);
	template <typename T> BError	cmdList(Node* n);
	BError				cmdList(Node* n);
	BError				cmdChange(Node* n);
	BError				cmdChangeResponse(Node* n);
	template <typename T> BError	cmdChange(Node* n);
	template <typename T> BError	cmdSplit(Node* n);
	BError				cmdSplit(Node* n);
	template <typename T> BError	cmdClone(Node* n);
	BError				cmdClone(Node* n);
	template <typename T> BError	cmdDelete(Node* n);
	
	BError		stationGetMembers(Station& obj, BDictString& members);
	BError		responseGetMembers(Response& obj, BDictString& members);
	BError		responseSetMembers(Response& obj, BDictString& members);
	BError		checkAndAddStation(BString station);
	BError		parseSelection(Node* n, Selection& sel);
	BError		parseValues(Node* n, BDictString& members);
	BError		setDefaultValues(BString type, BDictString& members);
	void		setVariable(BString name, BString value);
	BString		getVariable(BString name);
	BError		getVariableId(BString& value);
	void		printVarables();

private:
	AdminAccess&			obds;
	Parameters&			oparams;
	BDictString			ovariables;
	BArray<PoleZero>		opoleZeros;
	BArray<BArray<Fap> >		ofaps;
	BArray<Fir>			ofirs;
	BArray<BList<Response> >	oresponses;
	int				oline;
};

#endif
