/* %option   outfile="clex.cpp" header-file="clex.h" */

%{
/*******************************************************************************
 *	Scan.l		BEAM BdsMetadata Lexical analyser
 *			T.Barnaby, 	BEAM Ltd,	2011-06-05
 *******************************************************************************
 */
#include <stdio.h>
#include <string.h>
#include "Main.h"
#include "cgram.h"

extern Node*	yylval;
static void	comment0();
static void	comment1();
static void	nextLine();
static void	count();

#define	DOECHO	0
%}

D			[0-9]
DS			[0-9+\-]
L			[a-zA-Z_]
H			[a-fA-F0-9]
E			[Ee][+-]?{D}+
FS			(f|F|l|L)
IS			(u|U|l|L)*

%%
"/*"			{ comment0(); }
"//"			{ comment1(); return(COMMENT); }
"#"			{ comment1(); return(COMMENT); }
"\n"			{ nextLine(); }

"backup"		{ count(); return(BACKUP); }
"restore"		{ count(); return(RESTORE); }
"list"			{ count(); return(LIST); }
"info"			{ count(); return(INFO); }
"find"			{ count(); return(FIND); }
"add"			{ count(); return(ADD); }
"change"		{ count(); return(CHANGE); }
"split"			{ count(); return(SPLIT); }
"clone"			{ count(); return(CLONE); }
"set"			{ count(); return(SET); }
"print"			{ count(); return(PRINT); }
"clear"			{ count(); return(CLEAR); }
"delete"		{ count(); return(DELETE); }
"read"			{ count(); return(READ); }

"Station"		{ count(); return(TSTATION); }
"Location"		{ count(); return(TLOCATION); }
"Channel"		{ count(); return(TCHANNEL); }
"Digitiser"		{ count(); return(TDIGITISER); }
"Sensor"		{ count(); return(TSENSOR); }
"ChannelInstrument"	{ count(); return(TINSTRUMENT); }
"Calibration"		{ count(); return(TCALIBRATION); }
"Response"		{ count(); return(TRESPONSE); }
"Responses"		{ count(); return(TRESPONSES); }
"PoleZero"		{ count(); return(TPOLEZERO); }
"FAP"			{ count(); return(TFAP); }
"FIR"			{ count(); return(TFIR); }
"Source"		{ count(); return(TSOURCE); }
"Network"		{ count(); return(TNETWORK); }

{L}({L}|{D})*		{ count(); return(IDENTIFIER); }
0[xX]{H}+{IS}?		{ count(); return(CONSTANT); }
0{D}+{IS}?		{ count(); return(CONSTANT); }
{DS}?{D}+{IS}?		{ count(); return(CONSTANT); }
'(\\.|[^\\'])+'		{ count(); return(CONSTANT); }
{DS}?{D}+{E}{FS}?		{ count(); return(CONSTANT); }
{DS}?{D}*"."{D}+({E})?{FS}?	{ count(); return(CONSTANT); }
{DS}?{D}+"."{D}*({E})?{FS}?	{ count(); return(CONSTANT); }
\"(\\.|[^\\"])*\"	{ count(); return(STRING_LITERAL); }
{D}{D}{D}{D}-{D}{D}-{D}{D}T{D}{D}:{D}{D}:{D}{D}	{ count(); return(DATETIME); }
{D}{D}{D}{D}-{D}{D}-{D}{D}	{ count(); return(DATETIME); }
[.*]+			{ count(); return(REGEXP); }
$({L}|{D})*		{ count(); return(VARIABLE); }


";"			{ count(); return(';'); }
"{"			{ count(); return('{'); }
"}"			{ count(); return('}'); }
","			{ count(); return(','); }
"="			{ count(); return('='); }

[ \t\v\f]		{ count(); }
.			{ /* ignore bad characters */ }

%%

int	column	= 0;
int	line	= 1;

int yywrap()
{
	return 1;
}

static void comment0(){
	char	c, c1;

#if	DOECHO
loop:
	while ((c = yyinput()) != '*' && c != 0)
		putchar(c);

	if ((c1 = yyinput()) != '/' && c != 0)
	{
		unput(c1);
		goto loop;
	}

	if (c != 0)
		putchar(c1);
#else
loop:
	while ((c = yyinput()) != '*' && c != 0){
		if(c == '\n') line++;
	}

	if ((c1 = yyinput()) != '/' && c != 0) {
		unput(c1);
		goto loop;
	}
#endif
}

static void comment1(){
	char	c;
	BString	comment;

	while((c = yyinput()) != '\n' && c != 0){
		comment = comment + c;
	}

	if(c != 0){
		unput(c);
	}
	yylval = new Node(line, Node::TCOMMENT, comment);
}

static void nextLine(){
	column = 0;
	line++;
#if	DOECHO
	ECHO;
#endif
}

static void count()
{
	int i;

	for (i = 0; yytext[i] != '\0'; i++)
		if (yytext[i] == '\n')
			column = 0;
		else if (yytext[i] == '\t')
			column += 8 - (column % 8);
		else
			column++;

	yylval = new Node(line, Node::TNONE, yytext);
#if	DOECHO
	ECHO;
#endif
}
